import java.io.*;

public abstract class BenchBase {
    // input
    protected String filename;
    protected String encoding;
    protected boolean onMemoryMode;

    // output
    public int elementCount;
    public int attributeCount;
    public int characterCount;

    // internal
    private byte buffer[];

    protected abstract void parse(InputStream s) throws Exception;
    protected abstract InputStream createInputStream() throws Exception;

    private InputStream getStream() throws Exception {
	if (onMemoryMode) {
	    //System.err.println("onMemoryMode");
	    return new ByteArrayInputStream(buffer);
	} else {
	    return createInputStream();
	}
    }

    public void init(String name, String enc, boolean onMemory)
	throws Exception {
	filename = name;
	encoding = enc;
	onMemoryMode = onMemory;
	if (onMemoryMode) {
	    InputStream is = createInputStream();
	    int size = 0;
	    while (is.read() != -1) {
		size++;
	    }
	    buffer = new byte[size];
	    is = getClass().getResourceAsStream(filename);
	    is.read(buffer, 0, size);
	}
    }

    public long runWarmup() throws Exception {
	long start;
	long end;
	System.gc();
	start = System.currentTimeMillis();
	parse(getStream());
	end = System.currentTimeMillis();
	System.gc();
	return end - start;
    }

    public long runNormal(int repeat) throws Exception {
	long start;
	long end;
	System.gc();
	start = System.currentTimeMillis();
	for(int i=0; i<repeat; i++) {
	    parse(getStream());
	}
	end = System.currentTimeMillis();
	System.gc();
	return (end - start) / repeat;
    }

    public long runNoGC(int repeat) throws Exception {
	long start;
	long end;
	long totalTime = 0;
	// parse (with warmup gc)
	for(int i=0; i<repeat; i++) {
	    System.gc();
	    start = System.currentTimeMillis();
	    parse(getStream());
	    end = System.currentTimeMillis();
	    totalTime += end - start;
	}
	System.gc();
	return (totalTime) / repeat;
    }

}
    
